@@ -1,7 +1,7 @@
 /* boot.c  -  Boot image composition */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -28,6 +28,7 @@ source directory.
 #include "map.h"
 #include "partition.h"
 #include "boot.h"
+#include "loader.h"
 
 
 static GEOMETRY geo;
@@ -49,7 +50,11 @@ void boot_image(char *spec,IMAGE_DESCR *descr)
     int setup,fd,sectors;
     int modern_kernel;
 
-    if (verbose > 0) printf("Boot image: %s\n",spec);
+    if (verbose > 0) {
+	printf("Boot image: %s",spec);
+	show_link(spec);	/* in common.c */
+	printf("\n");
+    }
     fd = geo_open(&geo,spec,O_RDONLY);
     if (fstat(fd,&st) < 0) die("fstat %s: %s",spec,strerror(errno));
     if (read(fd,(char *) &buff,SECTOR_SIZE) != SECTOR_SIZE)
@@ -73,7 +78,10 @@ void boot_image(char *spec,IMAGE_DESCR *descr)
     else {
 	if (hdr.start % PAGE_SIZE)
 	    die("Can't load kernel at mis-aligned address 0x%08lx\n",hdr.start);
+#if 0
 	descr->start_page = hdr.start/PAGE_SIZE; /* load kernel high */
+#endif
+	descr->flags |= FLAG_LOADHI;	/* load kernel high */
     }
     geo_close(&geo);
     if (verbose > 1)
@@ -81,10 +89,18 @@ void boot_image(char *spec,IMAGE_DESCR *descr)
     if ((initrd = cfg_get_strg(cf_kernel,"initrd")) || (initrd = cfg_get_strg(
       cf_options,"initrd"))) {
 	if (!modern_kernel) die("Kernel doesn't support initial RAM disks");
-	if (verbose > 0) printf("Mapping RAM disk %s\n",initrd);
+	if (verbose > 0) {
+	    printf("Mapping RAM disk %s",initrd);
+	    show_link(initrd);
+	    printf("\n");
+	}
 	fd = geo_open(&geo,initrd,O_RDONLY);
 	if (fstat(fd,&st) < 0) die("fstat %s: %s",initrd,strerror(errno));
+#if 1
 	*(unsigned long *) descr->rd_size = st.st_size;
+#else
+	descr->rd_size = (st.st_size + SECTOR_SIZE - 1)/SECTOR_SIZE;
+#endif
 	map_begin_section();
 	map_add(&geo,0,(st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE);
 	sectors = map_end_section(&descr->initrd,0);
@@ -236,7 +252,13 @@ char *boot_mbr(const char *boot, int table)
 
 void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 {
-    int b_fd,l_fd,p_fd,walk,found,size;
+    int b_fd,p_fd,walk,found,size;
+#ifdef LCF_BUILTIN
+    BUILTIN_FILE *chain;
+    char *cname;
+#else
+    int l_fd;
+#endif
     unsigned short magic;
     BOOT_SECTOR buff[SETUPSECS-1];
     struct stat st;
@@ -245,6 +267,16 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     int letter = 0;
 
     if (!loader) loader = DFL_CHAIN;
+#ifdef LCF_BUILTIN
+    if (strstr(loader,"os2")) {
+	chain = &Os2_d;
+	cname = "OS/2";
+    }
+    else {
+	chain = &Chain;
+	cname = "CHAIN";
+    }
+#endif
     if (part && strlen(part)>0 && strlen(part)<=2) {
     	if (part[1]==0 || part[1]==':') {
     	    letter = toupper(part[0]);
@@ -258,9 +290,23 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     if (!part) part = boot_mbr(boot, 1);
     /* part may still be NULL */
 
-    if (verbose > 0)
-	printf("Boot other: %s%s%s, loader %s\n",boot,part ? ", on " : "",part
-	  ? part : "",loader);
+    if (verbose > 0) {
+#ifdef LCF_BUILTIN
+	printf("Boot other: %s%s%s, loader %s\n",
+		boot,
+		part ? ", on " : "",
+		part ? part : "",
+		cname);
+#else
+	printf("Boot other: %s%s%s, loader %s",
+		boot,
+		part ? ", on " : "",
+		part ? part : "",
+		loader);
+	show_link(loader);
+	printf("\n");
+#endif
+    }
 
 #ifdef LCF_AUTOAUTO
     if (!cfg_get_flag(cf_other, "change")) {
@@ -288,13 +334,20 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	    die("First sector of %s doesn't have a valid boot signature",boot);
     }
     memset(buff,0,sizeof(buff));
+#ifndef LCF_BUILTIN
     if ((l_fd = open(loader,O_RDONLY)) < 0)
 	die("open %s: %s",loader,strerror(errno));
     if ((size = read(l_fd,buff,sizeof(buff)+1)) < 0)
 	die("read %s: %s",loader,strerror(errno));
-    check_version(buff,STAGE_CHAIN);
     if (size > sizeof(buff))
 	die("Chain loader %s is too big",loader);
+    check_version(buff,STAGE_CHAIN);
+#else
+    size = chain->size;
+    if (size > sizeof(buff))
+	die("Chain loader %s is too big",loader);
+    memcpy(buff, chain->data, size);
+#endif
     if (!part) {
         p_fd = -1; /* pacify GCC */
         PART(buff,0).boot_ind = geo.device;
@@ -330,7 +383,9 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	if (!found) die("Partition entry not found.");
 	(void) close(p_fd);
     }
+#ifndef LCF_BUILTIN
     (void) close(l_fd);
+#endif
     buff[0].par_c.drive = geo.device;
     buff[0].par_c.head = letter ? letter : geo.device;
      		/* IBM boot manager passes drive letter in offset 0x25 */
@@ -348,5 +403,6 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	/* size is known */
     geo_close(&geo);
     if (verbose > 1)
-	printf("Mapped %d (%d+1+1) sectors.\n",SETUPSECS+2,SETUPSECS);
+	printf("Mapped %d (%d+1+1) sectors.\n",
+		SETUPSECS+SPECIAL_SECTORS, SETUPSECS);
 }
