@@ -1,7 +1,7 @@
 /* bsect.c  -  Boot sector handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -220,8 +220,6 @@ void bmp_do_table(char *cp, MENUTABLE *menu)
          menu->col + (MAX_IMAGE_NAME+1)*8 + (menu->ncol-1)*menu->xpitch > 640)
           && !nowarn)  
        fprintf(errstd,"Warning: 'bmp-table' may spill off screen\n");
-    image_menu_space = menu->ncol * menu->maxcol;
-    if (verbose>=3) printf("image_menu_space = %d\n", image_menu_space);
 }
 
 
@@ -467,6 +465,7 @@ static void open_bsect(char *boot_dev)
     if (read(fd,(char *) &bsect,SECTOR_SIZE) != SECTOR_SIZE)
 	die("read %s: %s",boot_dev ? boot_dev : dev.name,strerror(errno));
     bsect_orig = bsect;
+    ireloc = part_nowrite(boot_dev);
 }
 
 
@@ -681,7 +680,11 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
     scheme = cfg_get_strg(cf_options,"bitmap");
     if (message && scheme) die("'bitmap' and 'message' are mutually exclusive");
     bsect.par_1.msg_len = 0;
+#ifdef LCF_BUILTIN
+    bitmap = (loader==&Bitmap);
+#else
     bitmap = scheme ? 1 : 0;
+#endif
     if (bitmap) {
 	message = scheme;
 	if (!(stage_flags & STAGE_FLAG_BMP4)) {
@@ -851,6 +854,10 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	    fprintf(errstd,"Warning: 'bmp-table' not supported by boot loader\n");
     }
     bmp_do_table(scheme, menu);
+    if (bitmap) {
+	image_menu_space = menu->ncol * menu->maxcol;
+	if (verbose>=3) printf("image_menu_space = %d\n", image_menu_space);
+    }
     if ((scheme = cfg_get_strg(cf_options,"bmp-colors"))) {
 	if (!(stage_flags & STAGE_FLAG_BMP4) && !nowarn)
 	    fprintf(errstd,"Warning: 'bmp-colors' not supported by boot loader\n");
@@ -926,12 +933,23 @@ static int get_image(char *name,char *label,IMAGE_DESCR *descr)
 	if (image == MAX_IMAGES)
 	    die("Only %d image names can be defined",MAX_IMAGES);
 	if (image >= image_menu_space)
-	    die("'bmp-table=' has space for only %d images",
+	    die("Bitmap table has space for only %d images",
 	    			image_menu_space);
 	this_image = image++;
     }
     descrs.d.descr[this_image] = *descr;
     strcpy(descrs.d.descr[this_image].name,label);
+
+#ifdef LCF_VIRTUAL
+    if ( (deflt = cfg_get_strg(cf_options,"vmdefault")) &&
+#ifdef LCF_IGNORECASE
+		!strcasecmp(deflt,label))
+#else
+		!strcmp(deflt,label))
+#endif
+	descrs.d.descr[this_image].flags |= FLAG_VMDEFAULT;
+#endif
+
     return this_image;
 }
 
@@ -1008,13 +1026,14 @@ if (image) { /* long section specific to 'image=' */
     }
     check_options(options);
 
-    if (cfg_get_flag(cf_kernel,"lock") || cfg_get_flag(cf_options,"lock"))
+    if (cfg_get_flag(cf_kernel,"lock") || cfg_get_flag(cf_options,"lock")) {
 #ifdef LCF_READONLY
 	die("This LILO is compiled READONLY and doesn't support the LOCK "
 	  "option");
 #else
 	descr->flags |= FLAG_LOCK;
 #endif
+    }
 
     if ((cfg_get_flag(cf_options,"restricted") && 
              cfg_get_flag(cf_options,"mandatory")) ||
@@ -1041,6 +1060,14 @@ if (image) { /* long section specific to 'image=' */
 	}
 	descr->flags |= FLAG_PASSWORD;
     }
+
+#ifdef LCF_VIRTUAL
+    if (cfg_get_flag(cf_all,"vmwarn")) descr->flags |= FLAG_VMWARN;
+    if (cfg_get_flag(cf_all,"vmdisable")) descr->flags |= FLAG_VMDISABLE;
+    if ( (descr->flags & FLAG_VMWARN) && (descr->flags & FLAG_VMDISABLE) )
+	die ("VMWARN and VMDISABLE are not valid together");
+#endif
+
 #if 1
     if (cfg_get_flag(cf_all,"mandatory") || cfg_get_flag(cf_options,
       "mandatory")) {
@@ -1112,6 +1139,11 @@ static void bsect_done(char *name,IMAGE_DESCR *descr)
     if (verbose >= 0) {
 	printf("Added %s",descrs.d.descr[this_image].name);
 	if (alias) printf(" (alias %s)",alias);
+#ifdef LCF_VIRTUAL
+	if (descrs.d.descr[this_image].flags & FLAG_VMDEFAULT ||
+		(this>=0 && (descrs.d.descr[this].flags & FLAG_VMDEFAULT)) )
+	    printf(" @");
+#endif
 	if (this_image && this) putchar('\n');
 	else printf(" *\n");
     }
@@ -1136,7 +1168,6 @@ static void unbootable(void)
     fflush(stdout);
     fprintf(errstd,"\nWARNING: The system is unbootable !\n");
     fprintf(errstd,"%9sRun LILO again to correct this.","");
-    exit(1);
 }
 
 
@@ -1167,6 +1198,7 @@ void bsect_update(char *backup_file, int force_backup, int pass)
     struct stat st;
     char temp_name[PATH_MAX+1];
     int bck_file;
+    BOOT_SECTOR bsect_wr;
 
     if (!backup_file) {
 	sprintf(temp_name, BACKUP_DIR "/boot.%04X", boot_dev_nr);
@@ -1214,9 +1246,7 @@ void bsect_update(char *backup_file, int force_backup, int pass)
 	die("lseek %s: %s",boot_devnam ? boot_devnam : dev.name,
 	  strerror(errno));
     if (verbose > 0) printf("Writing boot sector.\n");
- /* failsafe check */
-    if (memcmp(bsect.sector+MAX_BOOT_SIZE, bsect_orig.sector+MAX_BOOT_SIZE, 64+8))
-    	die("LILO internal error:  Would overwrite Partition Table");
+
 #if 1
 /*    if ( MAJOR(boot_dev_nr)==MAJOR_FD  && */
     if (ireloc &&
@@ -1229,20 +1259,35 @@ void bsect_update(char *backup_file, int force_backup, int pass)
 	int len = bsect.par_1.code_length;
 	int space = BOOT_SIG_OFFSET - len;
 	space &= 0xFFF0;	/* roll back to paragraph boundary */
-	memcpy(&bsect_orig.sector[space], &bsect, len);
+        bsect_wr = bsect_orig;
+	memcpy(&bsect_wr.sector[space], &bsect, len);
 	if (space <= 0x80) {
-	    bsect_orig.sector[0] = 0xEB;		/* jmp short */
-	    bsect_orig.sector[1] = space - 2;
-	    bsect_orig.sector[2] = 0x90;		/* nop */
+	    bsect_wr.sector[0] = 0xEB;		/* jmp short */
+	    bsect_wr.sector[1] = space - 2;
+	    bsect_wr.sector[2] = 0x90;		/* nop */
 	} else {
-	    bsect_orig.sector[0] = 0xE9;		/* jmp near */
-	    *(short*)&bsect_orig.sector[1] = space - 3;
+	    bsect_wr.sector[0] = 0xE9;		/* jmp near */
+	    *(short*)&bsect_wr.sector[1] = space - 3;
 	}
-	bsect = bsect_orig;
+/***	bsect = bsect_orig;  ***/
 	if (verbose >= 1) printf("Boot sector relocation performed\n");
     }
+    else bsect_wr = bsect;
 #endif    	
-    if (write(fd, (char *)&bsect, SECTOR_SIZE) != SECTOR_SIZE)
+
+ /* failsafe check */
+#if 1
+    if (verbose >= 3) printf("Failsafe check:  boot_dev_nr = 0x%04x 0x%04x\n", boot_dev_nr, has_partitions(boot_dev_nr));
+#endif
+    if (
+      has_partitions(boot_dev_nr) &&
+      (P_MASK(boot_dev_nr)&boot_dev_nr)==0 &&
+      memcmp(bsect.sector+MAX_BOOT_SIZE, bsect_wr.sector+MAX_BOOT_SIZE, 64+8)
+      )
+    	die("LILO internal error:  Would overwrite Partition Table");
+ /* failsafe check */
+    	
+    if (write(fd, (char *)&bsect_wr, SECTOR_SIZE) != SECTOR_SIZE)
 	die("write %s: %s",boot_devnam ? boot_devnam : dev.name,
 	  strerror(errno));
     if (!boot_devnam) dev_close(&dev);
