@@ -61,6 +61,7 @@ CONFIG cf_options[] = {
   { cft_strg, RAID_EXTRA_BOOT,	NULL,		NULL,NULL },
   { cft_flag, "compact",	NULL,		NULL,NULL },
   { cft_strg, "default",	NULL,		NULL,NULL },
+  { cft_strg, "vmdefault",	NULL,		NULL,NULL },
   { cft_strg, "delay",		NULL,		NULL,NULL },
   { cft_strg, "timeout",	NULL,		NULL,NULL },
   { cft_flag, "prompt",		NULL,		NULL,NULL },
@@ -118,6 +119,8 @@ CONFIG cf_all[] = {
   { cft_strg, "fallback",	NULL,		NULL,NULL },
   { cft_flag, "single-key",	NULL,		NULL,NULL },
   { cft_strg, "literal",	NULL,		NULL,NULL },
+  { cft_flag, "vmdisable",	NULL,		NULL,NULL },
+  { cft_flag, "vmwarn",		NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_kernel[] = {
@@ -576,7 +579,7 @@ FILE *cfg_pw_open(void)
 #if 1
     if (stat(file_name, &buf)) die("Cannot stat '%s'", file_name);
     conf = buf.st_mtime;
-    if (!stat(pw_file_name, &buf) && conf>buf.st_mtime && !nowarn) {
+    if (!stat(pw_file_name, &buf) && conf>buf.st_mtime && !nowarn && !passw) {
         fprintf(errstd,"Warning: '%s' more recent than '%s'\n",
         	file_name, pw_file_name);
         fprintf(errstd," Running 'lilo -p' is recommended.\n");
