@@ -1,7 +1,7 @@
 /* common.c  -  Common data structures and functions. */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -20,8 +20,10 @@ source directory.
 #include "lilo.h"
 
 
+EXTRA extra;
 int verbose = 0,test = 0,compact = 0,linear = 0, raid_flags = 0, zflag = 0,
-      nowarn = 0, lba32 = 0, autoauto = 0, passw = 0, geometric = 0;
+      ireloc = 0, force_fs = 0,
+      nowarn = 0, lba32 = 0, autoauto = 0, passw = 0, geometric = 0, eflag = 0;
 
 unsigned short drv_map[DRVMAP_SIZE+1]; /* fixup maps ... */
 int curr_drv_map;
@@ -154,3 +156,18 @@ unsigned long crc32 (unsigned char *cp, int nsize, unsigned long polynomial)
    return ~crc;
 }
 
+
+/* show what a link resolves to */
+
+void show_link(char *name)
+{
+    int count;
+    char lname[1024];
+    
+    count = readlink(name, lname, sizeof(lname)-1);
+    if (count>0) {
+    	lname[count] = 0;
+    	printf(" -> %s", lname);
+    }
+}
+
