@@ -2,7 +2,7 @@
 /* common.h  -  Common data structures and functions. */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2000 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -50,14 +50,11 @@ source directory.
 						id_name:	.blkb	MAX_IMAGE_NAME_asm+1
 ;*/    unsigned short password_crc[MAX_PW_CRC*(sizeof(long)/sizeof(short))];  /* 4 password CRC-32 values
 						id_password_crc:.blkb	MAX_PW_CRC_asm*4
-;*/    unsigned short rd_size[2]; /* RAM disk size in bytes, 0 if none
+;*/    unsigned short rd_size[2]; /* RAM disk size in sectors, 0 if none
 						id_rd_size:	.blkb	4		;don't change the order !!!
 ;*/    SECTOR_ADDR initrd,start;  /* start of initrd & kernel
 						id_initrd:	.blkb	sa_size		;  **
 						id_start:	.blkb	sa_size		;  **
-;*/    unsigned short start_page; /* page at which the kernel is loaded high, 0
-;*/				  /* if loading low
-						id_start_page:	.blkb	2		;  **
 ;*/    unsigned short flags,vga_mode; /* image flags & video mode
 						id_flags:	.blkb	2		;  **
 						id_vga_mode:	.blkb	2		;  **
@@ -71,10 +68,8 @@ source directory.
 							block	0
 ;*/    unsigned char cli;     /* clear interrupt flag instruction
 						par1_cli:	.blkb	1
-;*/    unsigned char call_ins;  /* call instruction opcode
-						par1_call_ins:	.blkb	1
-;*/    short call_offset;  /* offset to destination of call instruction
-						par1_call_offs:	.blkb	2
+;*/    unsigned char jmp0, jmp1, nop;	/* short jump followed by a NOP
+						par1_jump:	.blkb	3
 ;*/    unsigned short code_length;  /* length of the first stage code
 						par1_code_len:	.blkb	2
 ;*/    char signature[4]; /* "LILO"
@@ -95,12 +90,13 @@ source directory.
 						par1_timeout:	.blkb	2
 ;*/    unsigned short delay; /* delay: wait that many 54 msec units.
 						par1_delay:	.blkb	2
+;*/    unsigned short dataend; /* full size of second-stage, including parameter line
+						par1_dataend:	.blkb	2
 ;*/    unsigned short msg_len; /* 0 if none
 						par1_msg_len:	.blkb	2
 ;*/    SECTOR_ADDR msg; /* initial greeting message
 						par1_msg:	.blkb	sa_size
-;*/    SECTOR_ADDR descr[MAX_DESCR_SECS+1]; /* 2 descriptors and default command line
-						par1_descr:	.blkb	sa_size*MAX_DESCR_SECS_asm
+;*/    SECTOR_ADDR dflcmd; /* default command line
 						par1_dflcmd:	.blkb	sa_size
 ;*/    unsigned char prompt; /* FLAG_PROMPT=always, FLAG_RAID install
 						par1_prompt:	.blkb	1
@@ -152,8 +148,19 @@ source directory.
 ;*/	short t_row, t_col;		/* timer position
 						mt_t_row:	.blkw	1
 						mt_t_col:	.blkw	1
+;*/	short mincol, reserved[3];	/* BMP min per col before spill to next, reserved spacer
+						mt_mincol:	.blkw	1
+								.blkw	3
+;*/	char unused[160-MAX_DESCR_SECTORS*sizeof(SECTOR_ADDR)];		/* spacer to Descriptor ptrs
+						mt_unused:	.blkb	160-sa_size*MAX_DESCR_SECTORS_asm
+;*/	SECTOR_ADDR mt_descr[MAX_DESCR_SECTORS];	/* descriptor disk addresses
+						mt_descr:	.blkb	sa_size*MAX_DESCR_SECTORS_asm
+;*/	short checksum[2];		/* checksum longword
+						mt_cksum:	.blkw	2
 ;*/	unsigned char mt_flag;		/* contains the FLAG_NOBD only
 						mt_flag:	.blkb	1
+;*/	char unused2[3];		/* spacer beyond checksum
+						mt_unused2:	.blkb	3
 ;*/} MENUTABLE;		/* MENU and BITMAP parameters at KEYTABLE+256
 						mt_size:
 							endb
@@ -178,10 +185,33 @@ typedef struct {
     unsigned char ptable[PARTITION_ENTRY*PARTITION_ENTRIES]; /* part. table */
 } BOOT_PARAMS_C; /* chain loader */
 
+typedef struct {
+    unsigned char  bytes_per_sector[2];
+    unsigned char  sectors_per_cluster;
+    unsigned char  reserved_sectors[2];
+    unsigned char  number_of_FATs;
+    unsigned char  root_dir_entries[2];
+    unsigned char  total_sectors[2];
+    unsigned char  media_descriptor;
+    unsigned char  sectors_per_FAT[2];
+    unsigned char  sectors_per_track[2];	/* DOS v.3 */
+    unsigned char  heads_per_cylinder[2];	/* DOS v.3 */
+    unsigned char  hidden_sectors[4];
+    unsigned char  total_sectors_long[4];	/* if total_sectors==0 */
+    unsigned char  reserved[7];		/* pad to 32 bytes */
+} BIOS_PARAMETER_BLOCK;
+
+typedef struct {
+    unsigned char jump[3];	/* jump to boot code */
+    char system[8];		/* system ID */
+    BIOS_PARAMETER_BLOCK bpb;	/* BIOS parameter block */
+} BOOT_PARAMS_DOS;	/* DOS fat header */
+
 typedef union {
     BOOT_PARAMS_1 par_1;
     BOOT_PARAMS_2 par_2;
     BOOT_PARAMS_C par_c;
+    BOOT_PARAMS_DOS par_d;
     unsigned char sector[SECTOR_SIZE];
 } BOOT_SECTOR;
 
@@ -195,6 +225,12 @@ typedef union {
     	unsigned long checksum;
     } l;
 } DESCR_SECTORS;
+
+typedef struct {
+    int size;
+    unsigned char data[1];
+} BUILTIN_FILE;
+
 #endif
 /*
 IMAGES_numerator = SECTOR_SIZE_asm*MAX_DESCR_SECS_asm - 4 - 1
@@ -214,14 +250,20 @@ typedef struct {
     unsigned long ramdisk;	/* 24: RAM disk start address */
     unsigned long ramdisk_size;	/* 28: RAM disk size */
     unsigned short b,c;		/* 32: bzImage hacks */
-    unsigned short heap_end_ptr;/* 36: end of free area after setup code */
+    unsigned short heap_end_ptr;/* 36: 2.01 end of free area after setup code */
+    unsigned char d;		/* 38: padding */
+    unsigned long cmd_line_ptr; /* 40: 2.02 address32 of command line */
+    unsigned long ramdisk_max;	/* 44: 2.03 address32 of highest mem. for ramdisk */
 } SETUP_HDR;
 
 #define alloc_t(t) ((t *) alloc(sizeof(t)))
 
+typedef enum {X_NULL=0, X_NONE, X_AUTO, X_MBR_ONLY, X_SPEC} EXTRA;
 
+extern EXTRA extra;
 extern int verbose,test,compact,linear,nowarn,lba32,autoauto,passw,geometric;
-extern int boot_dev_nr,raid_flags,do_md_install,zflag;
+extern int ireloc, force_fs;
+extern int boot_dev_nr,raid_flags,do_md_install,zflag,eflag;
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
 extern int curr_drv_map;
 extern unsigned long prt_map[PRTMAP_SIZE+1];
@@ -234,49 +276,45 @@ extern FILE *errstd;
 
 
 /*volatile*/ void pdie(char *msg);
-
 /* Do a perror and then exit. */
 
 
 /*volatile*/ void die(char *fmt,...);
-
 /* fprintf an error message and then exit. */
 
 
 void *alloc(int size);
-
 /* Allocates the specified number of bytes. Dies on error. */
 
 
 void *ralloc(void *old,int size);
-
 /* Changes the size of an allocated memory area. Dies on error. */
 
 
 char *stralloc(const char *str);
-
 /* Like strdup, but dies on error. */
 
 
 int to_number(char *num);
-
 /* Converts a string to a number. Dies if the number is invalid. */
 
 
 void check_version(BOOT_SECTOR *sect, int stage);
-
 /* Verify that a boot sector has the correct version number. */
 
 
 int stat_equal(struct stat *a, struct stat *b);
-
 /* Compares two stat structures. Returns a non-zero integer if they describe
    the same file, zero if they don't. */
 
 
 unsigned long crc32 (unsigned char *cp, int nsize, unsigned long polynomial);
-
 /* calculate a CRC-32 polynomial */
 
+
+void show_link(char *name);
+/* show what a link resolves to */
+
 #endif
 #endif
+
