@@ -1,6 +1,13 @@
 /* device.c  -  Device access */
+/*
+Copyright 1992-1997 Werner Almesberger.
+Copyright 1999-2001 John Coffman.
+All rights reserved.
 
-/* Copyright 1992-1997 Werner Almesberger. See file COPYING for details. */
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 
 
 #include <unistd.h>
@@ -99,7 +106,7 @@ static void cache_add(const char *name,int number)
 {
     CACHE_ENTRY *entry;
 
-    if (verbose >= 5) printf("Caching device %s (0x%04X)\n",name,number);
+    if (verbose >= 4) printf("Caching device %s (0x%04X)\n",name,number);
     entry = alloc_t(CACHE_ENTRY);
     entry->name = stralloc(name);
     entry->number = number;
@@ -166,34 +173,93 @@ void dev_close(DEVICE *dev)
     free(dev->name);
 }
 
+#define MAX 15
+
+void cache_scsi (char *name, int major)
+{
+    char tem[12], format[12];
+    int i, j, dev;
+    
+    for (i=15; i>=0; i--) {
+	dev = (major<<8) | (i<<4);
+	strcpy(format, name);
+	format[7] += i;
+	cache_add(format, dev);
+	strcat(format, "%d");
+	for (j = MAX; j; j--) {
+	    sprintf(tem, format, j);
+	    cache_add(tem, dev | j);
+	}
+    }
+}
+
+
+void cache_ide (char *name, int major)
+{
+    char tem[12];
+    char format[12];
+    int i, dev, minor;
+    
+    minor = name[7]&1 ? 0 : 64;
+    strcat(strcpy(format, name), "%d");
+    dev = (major<<8) | minor;
+    for (i = MAX; i; i--) {
+        sprintf(tem, format, i);
+        cache_add(tem, dev | i);
+    }
+    cache_add(name, dev);
+}
+
 
 void preload_dev_cache(void)
 {
     char tmp[12];
-    int i;
+    int i, vsave;
+
+#define QUIET 1
+
+#ifdef QUIET
+    vsave = verbose;
+    verbose -= QUIET;	/* quiet useless output */
+#endif
+
+    cache_add("/dev/fd0", 0x0200);
+    cache_add("/dev/fd1", 0x0201);
+    
+#if 0
+    cache_ide("/dev/hdt", MAJOR_IDE10);
+    cache_ide("/dev/hds", MAJOR_IDE10);
+    cache_ide("/dev/hdr", MAJOR_IDE9);
+    cache_ide("/dev/hdq", MAJOR_IDE9);
+
+    cache_ide("/dev/hdp", MAJOR_IDE8);
+    cache_ide("/dev/hdo", MAJOR_IDE8);
+    cache_ide("/dev/hdn", MAJOR_IDE7);
+    cache_ide("/dev/hdm", MAJOR_IDE7);
+    
+    cache_ide("/dev/hdl", MAJOR_IDE6);
+    cache_ide("/dev/hdk", MAJOR_IDE6);
+    cache_ide("/dev/hdj", MAJOR_IDE5);
+    cache_ide("/dev/hdi", MAJOR_IDE5);
+#endif
+    
+    cache_scsi("/dev/sda", MAJOR_SD);
+    
+    cache_ide("/dev/hdh", MAJOR_IDE4);
+    cache_ide("/dev/hdg", MAJOR_IDE4);
+    cache_ide("/dev/hdf", MAJOR_IDE3);
+    cache_ide("/dev/hde", MAJOR_IDE3);
 
-    cache_add("/dev/hda",0x300);
-    for (i = 1; i <= 8; i++) {
-	sprintf(tmp,"/dev/hda%d",i);
-	cache_add(tmp,0x300+i);
-    }
-    cache_add("/dev/hdb",0x340);
-    for (i = 1; i <= 8; i++) {
-	sprintf(tmp,"/dev/hdb%d",i);
-	cache_add(tmp,0x340+i);
-    }
-    cache_add("/dev/sda",0x800);
-    for (i = 1; i <= 8; i++) {
-	sprintf(tmp,"/dev/sda%d",i);
-	cache_add(tmp,0x800+i);
-    }
-    cache_add("/dev/sdb",0x810);
-    for (i = 1; i <= 8; i++) {
-	sprintf(tmp,"/dev/sdb%d",i);
-	cache_add(tmp,0x810+i);
-    }
     for (i = 0; i <= 7; i++) {
 	sprintf(tmp,"/dev/loop%d",i);
 	cache_add(tmp,0x700+i);
     }
+
+    cache_ide("/dev/hdd", MAJOR_IDE2);
+    cache_ide("/dev/hdc", MAJOR_IDE2);
+    cache_ide("/dev/hdb", MAJOR_HD);
+    cache_ide("/dev/hda", MAJOR_HD);
+#ifdef QUIET
+    verbose = vsave;
+#endif
 }
