@@ -10,7 +10,7 @@
 #include <bios.h>
 #include "../bdata.h"
 
-#define DISK_VERSION "2.0"
+#define DISK_VERSION "2.1.1"
 
 #if __MSDOS__==0
 #define putch bios_putc
@@ -49,6 +49,8 @@ struct SREGS sreg;
 int last_good_disk;
 int video_1 = 0xF00;
 int num_hd = BD_MAX_HARD;
+int errno;
+dword hma;	/* highest memory address */
 struct {
   long	start, start_hi,
 	length, length_hi,
@@ -56,6 +58,68 @@ struct {
     } mem_map;
 #define E820_MAGIC 0x534D4150
 
+struct gdt_entry {
+	unsigned short limit;
+	unsigned short base01;
+	unsigned char  base2;
+	unsigned char  dtype;	/* 0x93 for data */
+	unsigned char  limit2;	/* limit in low nibble, granularity & 32-bit in high nibble */
+	unsigned char  base3;
+};
+
+struct gdt_entry gdt[6];
+
+static
+int hicopy (unsigned long to, unsigned long from, int wcount)
+{
+    int status;
+    unsigned char save;
+    
+    memset(gdt, 0, sizeof(gdt));
+    gdt[2].limit = gdt[3].limit = 0xFFFF;
+    gdt[2].dtype = gdt[3].dtype = 0x93;
+    
+    gdt[2].base01 = from;
+    gdt[2].base2 = from>>16;
+    gdt[2].base3 = from>>24;
+    
+    gdt[3].base01 = to;
+    gdt[3].base2 = to>>16;
+    save = gdt[3].base3 = to>>24;
+    
+    segread(&sreg);
+    sreg.es = sreg.ds;
+    reg.h.ah = 0x87;
+    reg.x.cx = wcount;
+    reg.x.si = gdt;
+/***    gdt[3].base3 &= 0;   / crosstalk */
+    int86x(0x15, &reg, &oreg, &sreg);
+    
+    status = oreg.h.ah;
+    if (oreg.x.cflag) status |= 0x100;
+    if (save!=gdt[3].base3) status |= 0x200;
+    errno |= status;
+    return status;
+}
+
+unsigned long linear(void *ptr)
+{
+    segread(&sreg);
+    return ((unsigned long)sreg.ds<<4) + (unsigned int)ptr;
+}
+
+word hipeekw(long address)
+{
+    word temp;
+    hicopy(linear(&temp), address, 1);
+    return temp;
+}
+
+int hipokew(long address, word value)
+{
+    return hicopy(address, linear(&value), 1);
+}
+
 #if __MSDOS__==0
 static
 bios_putc0(int c)
@@ -161,7 +225,7 @@ static
 void banner(char *version)
 {
 	printf(	"\n\n\n"
-">>>> Disk Detection and Parameter Display <<<<\n"
+">>>> Disk Detection and Parameter Display <<<<\n\n\n"
 "Version %s, Copyright (C) 1999-2002  John Coffman <johninsd@san.rr.com>\n"
 "Portions Copyright (C) 1996-2001 Robert de Bath, used with permission\n"
 "Re-use and redistribution rights set forth in the file \"COPYING\".\n\n",
@@ -365,10 +429,73 @@ void get_conv_mem(void)
     printf("\n(The A20 line is %sabled.)\n", a20() ? "en" : "dis");
 }
 
+static
+void mov_ext_mem(void)
+{
+    word status, temp, vtemp;
+    dword high, veryhigh;
+        
+    pause();
+    segread(&sreg);
+    sreg.es = sreg.ds;
+    printf(SPACER
+    		"Int 15h  Function 87h\t\t[AT][PS/2]\n"
+    		"Move Extended Memory Block\n\n"
+    		"Call With:\n    ");
+    		print_sregs(&sreg);
+    		reg.x.ax = 0x8700;
+    		reg.x.cx = 1;
+    		reg.x.si = &gdt;
+    		printf("    ");
+    		print_regs(&reg);
+    		
+    	high = 1024L*(1024L+128)-2;	/* 1Mb + 128K */
+    	veryhigh = high+16L*1024L*1024L;
+    	if (veryhigh >= hma) veryhigh=0;
+#define WORDA 0xA5C6
+#define WORDB 0x6CA5
+
+	errno = 0;
+	temp = hipeekw(high);
+	status = hipokew(high, WORDA^temp);
+    	printf("\nReturns:\n    ");
+    	print_sregs(&sreg);
+    	printf("    ");
+    	print_regs(&oreg);
+    	print_carry((status>>8) & 1);
+
+	printf("\nR/W test at address %08lx ", high);
+	if (hipeekw(high) != (WORDA^temp)) errno |= 0x400;
+	hipokew(high, temp);
+	if (hipeekw(high) != temp) errno |= 0x800;
+	printf("%ssuccessful\n", errno ? "un" : "");
+	if (errno) printf("Error code = 0x%04x\n", errno);
+	
+	if (veryhigh) {
+		printf("R/W test at address %08lx ", veryhigh);
+		vtemp = hipeekw(veryhigh);
+		hipokew(veryhigh, WORDB^vtemp);
+		if (hipeekw(high) != temp) errno |= 0x200;
+		if (hipeekw(veryhigh) != (WORDB^vtemp)) errno |= 0x400;
+		hipokew(veryhigh, vtemp);
+		if (hipeekw(high) != temp) errno |= 0x200;
+		if (hipeekw(veryhigh) != vtemp) errno |= 0x800;
+		printf("%ssuccessful\n", errno ? "un" : "");
+		if (errno) printf("Error code = 0x%04x\n", errno);
+	}
+    	if (errno & 0xE00)
+    	    printf("\nThere is crosstalk between the two addresses\n"
+    	    	"The function does not support full 386 32-bit addressing.\n");
+}
+
+#define NAREA 32
+
 static
 void get_ext_mem(void)
 {
     long b, b1;
+    dword t;
+    int i;
     
     pause();
     printf(SPACER
@@ -378,18 +505,52 @@ void get_ext_mem(void)
     		"    EAX=0000E820  EBX=00000000  ECX=00000014  EDX=%lx\n\n",
     		E820_MAGIC );
 
-    b = e820(b1=0);
+    b = e820(b1=i=0);
     if (b > 0) {
+	dword start[NAREA], length[NAREA];
+	int j, k, ovlap;
 
-    /*             00    000000000000   000000000000   (1) avail	*/
-	printf("\nEBX        Start         Length      Type\n\n");
+    /*           00    000000000000   000000000000   (1) avail	*/
+	printf("EBX        Start         Length      Type\n\n");
 	do {
-	printf(" %02lx    %04hx%08lx   %04hx%08lx   (%d) %s\n", b1,
+	    printf(" %02lx    %04hx%08lx   %04hx%08lx   (%d) %s\n", b1,
 		(short)mem_map.start_hi, mem_map.start,
 		(short)mem_map.length_hi, mem_map.length, (int)mem_map.mtype,
-		mem_map.mtype == 1 ? "available" : "reserved");
-	b = e820(b1=b);
+		mem_map.mtype == 1 ? "available" :
+		mem_map.mtype == 2 ? "reserved"  :
+		mem_map.mtype == 3 ? "ACPI" :
+		mem_map.mtype == 4 ? "NVS"  :  "unknown-reserved");
+	    if (mem_map.mtype==1 && mem_map.start_hi==0 && mem_map.start<=1024L*1024L) {
+		if (mem_map.length_hi==0) hma = mem_map.start+mem_map.length;
+		else hma = 0xFFF00000L;
+	    }
+	    if (i < NAREA) {
+		start[i] = *(dword*)(((char*)&mem_map.start)+1);
+		length[i] = *(dword*)(((char*)&mem_map.length)+1);
+	    }
+	    i++;
+	    b = e820(b1=b);
 	} while (b1 > 0);
+	printf("\n");
+	if (i > NAREA) {
+	    i = NAREA;
+	}
+	ovlap = 0;
+	for (k=0; k<i-1; k++) {
+	    dword s, e;
+	    s = start[k];
+	    e = s + length[k];
+	    for (j=k+1; j<i; j++) {
+	    	dword ss, ee;
+	    	ss = start[j];
+	    	ee = ss + length[j];
+		if (!(ss < s && ee <= s || ss >= e && ee > e)) {
+		    printf("*** Memory areas %d and %d overlap ***\n", k, j);
+		    ovlap++;
+		}
+	    }
+	}
+	if (!ovlap) printf("No memory areas overlap\n");
     } else {
 	printf("Returns:\n");
 	if (b==-1) print_carry(1);
@@ -415,7 +576,12 @@ void get_ext_mem(void)
     if (!oreg.x.cflag) {
 	printf("\nNumber of 1K blocks between 1M and 16M:  %u\n", oreg.x.ax);
 	printf(  "Number of 64K blocks above 16M:  %u\n", oreg.x.bx);
+	t = 1024L*(oreg.x.ax+1024);
+	t += 64L*1024L*oreg.x.bx;
+	if (!hma) hma = t;
+	else if (hma!=t) printf("A different amount of memory is reported by this function\n");
     }
+    else printf("\nFunction is not supported.\n");
 
     pause();
     printf(SPACER
@@ -431,7 +597,8 @@ void get_ext_mem(void)
     decimal( (unsigned long)reg.x.ax );
     printf("K of extended memory.\n");
 
-
+    t = (reg.x.ax + 1024L) * 1024L;
+    if (!hma) hma = t;
 }    
     
 static
@@ -878,7 +1045,9 @@ void main(void)
 #endif
     get_equip_cfg();
     get_conv_mem();
+    hma = 0;
     get_ext_mem();
+    if (hma>1024L*1024L) mov_ext_mem();
     m = get_video_mode();
     if (m) m = get_cfg_info();
     if (m) m = get_comb_code();
