@@ -19,6 +19,11 @@ source directory.
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+
+#ifdef	_SYS_STATFS_H
+#define	_I386_STATFS_H	/* two versions of statfs is not good ... */
+#endif
+
 #include "config.h"
 #include "lilo.h"
 #include "common.h"
@@ -574,6 +579,7 @@ static void edit_colors(void)
 		break;
 	    case 'H':
 		get3colors("Highlight text", &mn.h_fg);
+		break;
 	    case 'T':
 		if (timer) get3colors("Timer text", &mn.t_fg);
 		else goto bad;
