@@ -63,6 +63,27 @@ struct lv_bmap {
 #endif
 #endif
 
+#ifdef LCF_EVMS
+struct evms_get_bmap_t {
+    __u64 rsector;
+    __u32 dev;
+    int status;
+};
+
+struct evms_version_t {
+    __u32 major;
+    __u32 minor;
+    __u32 patch;
+};
+
+#ifndef EVMS_GET_BMAP
+#define EVMS_GET_BMAP		_IOWR(MAJOR_EVMS, 0xC7, struct evms_get_bmap_t)
+#endif
+#ifndef EVMS_GET_IOCTL_VERSION
+#define EVMS_GET_IOCTL_VERSION	_IOR(MAJOR_EVMS, 0x0, struct evms_version_t)
+#endif
+#endif
+
 #ifndef HDIO_GETGEO
 #define HDIO_GETGEO HDIO_REQ
 #endif
@@ -201,7 +222,11 @@ void do_disk(void)
 	}
     if (cfg_get_flag(cf_disk,"inaccessible")) {
 	entry->heads = 0;
-	if (cfg_get_strg(cf_disk,"bios") || cfg_get_strg(cf_disk,"sectors") ||
+	if (
+#if 0
+		cfg_get_strg(cf_disk,"bios") || 
+#endif
+		cfg_get_strg(cf_disk,"sectors") ||
 	  cfg_get_strg(cf_disk,"heads") || cfg_get_strg(cf_disk,"cylinders"))
 	    die("No geometry variables allowed if INACCESSIBLE");
     }
@@ -345,6 +370,50 @@ void lvm_bmap(struct lv_bmap *lbm)
 #endif
 
 
+#ifdef LCF_EVMS
+void evms_bmap(struct evms_get_bmap_t *ebm)
+{                                  
+    DEVICE dev;
+    static int evms_fd = -1;
+    static dev_t evms_last_dev = 0;
+
+    if (ebm->dev != evms_last_dev) {
+        char evms_blk[] = "/dev/evms/block_device";
+        struct evms_version_t evms_ver;
+
+        // Open the EVMS device
+        if (evms_fd != -1)
+            close(evms_fd);
+
+        evms_fd = open(evms_blk, O_RDONLY);
+        if (evms_fd < 0)
+            die("Can't open EVMS block device %s.\n", evms_blk);
+
+        // Get EVMS ioctl version number.
+        if (ioctl(evms_fd, EVMS_GET_IOCTL_VERSION, &evms_ver) < 0)
+            die("EVMS_GET_IOCTL_VERSION failed on %s.\n", evms_blk);
+
+        // Check that the ioctl version is >= 7.1.0
+        if (evms_ver.major < 7 ||
+            (evms_ver.major == 7 && evms_ver.minor < 1))
+            die("EVMS ioctl version %d.%d.%d does not support booting.\n",
+                evms_ver.major, evms_ver.minor, evms_ver.patch);
+        close(evms_fd);
+
+        evms_fd = dev_open(&dev, ebm->dev, O_RDONLY);
+        if (evms_fd < 0)
+            die("Can't open EVMS block device %#x\n", ebm->dev);
+        evms_last_dev = ebm->dev;
+    }
+
+    if (ioctl(evms_fd, EVMS_GET_BMAP, ebm) < 0) {
+        perror(__FUNCTION__);
+        pdie("EVMS_GET_BMAP error or ioctl unsupported. Can't have image on EVMS volume.\n");
+    }
+}
+#endif
+
+
 static void geo_query_dev(GEOMETRY *geo,int device,int all)
 {
     DEVICE dev;
@@ -622,6 +691,21 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	device = geo->base_dev = lbmA.lv_dev;
     }
 #endif
+
+#ifdef LCF_EVMS
+    if (MAJOR(device) == MAJOR_EVMS) {
+        struct evms_get_bmap_t ebm;
+        
+        ebm.rsector = 0;
+        ebm.dev = device;
+        ebm.status = 0;
+        
+        evms_bmap(&ebm);
+        
+        device = geo->base_dev = ebm.dev;
+    }
+#endif
+
     /* Find underlying device for MD RAID */
     if (MAJOR(device) == MD_MAJOR) {
         char mdxxx[16];
@@ -661,11 +745,15 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     if (inherited)
 	for (walk = disktab; walk; walk = walk->next)
 	    if (walk->device == (device & D_MASK(device))) break;
-    if (walk && !walk->heads)
+
+/* add 'all' to conditional below -- JRC 2002-08-20 */
+    if (walk && !walk->heads && all)
 	die("Device 0x%04X: Configured as inaccessible.\n",device);
     keep_cyls = !walk || walk->bios == -1 || walk->heads == -1 ||
       walk->sectors == -1 || inherited || walk->start == -1;
-    if (keep_cyls) {
+
+/* add 'all' to conditional below -- JRC 2002-08-20 */
+    if (keep_cyls && (all || MAJOR(device)==MAJOR_FD) ) {
 	geo_query_dev(geo,device,all);
 	if (!nowarn && (geo->device & 0x7f) >= bios_max_devs() &&
 	  user_device == -1 && (!walk || walk->bios == -1))
@@ -693,6 +781,9 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     if (geo->heads > BIOS_MAX_HEADS)
 	die("Device 0x%04X: Maximum number of heads is %d, not %d\n",device,
 	  BIOS_MAX_HEADS,geo->heads);
+    if (geo->heads == BIOS_MAX_HEADS && !nowarn)
+	fprintf(errstd, "Warning: Maximum number of heads = %d (as specified)\n"
+			"   exceeds standard BIOS maximum of 255.\n", geo->heads);
     if (geo->sectors > BIOS_MAX_SECS)
 	die("Device 0x%04X: Maximum number of sectors is %d, not %d\n",
 	  device,BIOS_MAX_SECS,geo->sectors);
@@ -763,9 +854,17 @@ int geo_open_boot(GEOMETRY *geo,char *name)
     if (!S_ISREG(st.st_mode) && !S_ISBLK(st.st_mode))
 	die("%s: neither a reg. file nor a block dev.",name);
     geo->dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
+#if 0
     if (MAJOR(geo->dev) == MAJOR_FD) geo->fd = 0;
     else if ((geo->fd = open(name,O_NOACCESS)) < 0)
 	    die("open %s: %s",name,strerror(errno));
+#else
+    if (MAJOR(geo->dev) != MAJOR_FD) {
+	if ((P_MASK(geo->dev) & geo->dev) != 0)
+	    die("UNSAFE may be used with floppy or MBR only");
+    }
+    geo->fd = 0;
+#endif
     geo_get(geo, geo->dev, -1, 0);
     geo->file = S_ISREG(st.st_mode);
     geo->boot = 1;
@@ -824,8 +923,28 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
 	block = lbm.lv_block;
     }
 #endif
+
+#ifdef LCF_EVMS
+    if (MAJOR(geo->dev) == MAJOR_EVMS) {
+        struct evms_get_bmap_t ebm;
+                          
+        ebm.rsector = block * geo->spb;
+        ebm.dev = geo->dev;
+        ebm.status = 0;
+                            
+        evms_bmap(&ebm);
+        if (ebm.dev != geo->base_dev)
+            die("EVMS boot volume cannot be on multiple disks.\n");
+        sector = ebm.rsector + ((offset/SECTOR_SIZE) % geo->spb) + geo->start;
+    }
+    else {
+        sector = block*geo->spb+((offset/SECTOR_SIZE) % geo->spb);
+        sector += geo->start;
+    }
+#else
     sector = block*geo->spb+((offset/SECTOR_SIZE) % geo->spb);
     sector += geo->start;
+#endif
  /*   Always use CHS addressing on floppies:     JRC   */
     if ((geo->device & 0x80) && (linear || lba32)) {
         addr->device = geo->device | (linear ? LINEAR_FLAG : (LBA32_FLAG|LBA32_NOCOUNT))
