@@ -1,6 +1,14 @@
 /* identify.c  -  Translate label names to kernel paths */
 
-/* Copyright 1992-1995 Werner Almesberger. See file COPYING for details. */
+/* 
+Copyright 1992-1998 Werner Almesberger.
+Copyright 1999-2002 John Coffman.
+All rights reserved.
+
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 
 
 #include <stdlib.h>
@@ -17,13 +25,14 @@ static char *identify,*opt;
 
 static void do_identify(char *var,char type)
 {
-    char *label,*path,*alias,*initrd;
-    int image, ramdisk;
+    char *label,*path,*alias,*initrd,*keytab;
+    int image, ramdisk,kt;
 
 #if 1
     image = !!strchr(opt,'i');
     ramdisk = !!strchr(opt,'r');
-    if (opt && !image && !ramdisk) exit(1);
+    kt = !!strchr(opt,'k');
+    if (opt && !image && !ramdisk && !kt) exit(1);
     if (!opt) image = 1;
 #else
     image = ramdisk = 1;
@@ -36,6 +45,8 @@ static void do_identify(char *var,char type)
     else if (!label) label = path;
     alias = cfg_get_strg(cf_all,"alias");
     initrd = cfg_get_strg(cf_kernel,"initrd");
+    keytab = cfg_get_strg(cf_options,"keytable");
+    if (!keytab) keytab="us.ktl";
 #ifdef LCF_IGNORECASE
     if (!strcasecmp(label,identify) || (alias && !strcasecmp(alias,identify))) {
 #else
@@ -43,6 +54,7 @@ static void do_identify(char *var,char type)
 #endif
 	if (image) printf("%s\n",path);
 	if (ramdisk) printf("%s\n",initrd?initrd:"No initial ramdisk specified");
+	if (kt) printf("%s\n",keytab);
 	exit(0);
     }
 }
