@@ -626,8 +626,13 @@ static void show_images(char *map_file)
     }
     for (image = 0; image < MAX_IMAGES; image++)
 	if (*(name = descrs.d.descr[image].name)) {
-	    printf("%s%-" S(MAX_IMAGE_NAME) "s%s",verbose > 0 ? "  " : "",name,
-	      image ? "  " : " *");
+	    printf("%s%-" S(MAX_IMAGE_NAME) "s %s%s",verbose > 0 ? "  " : "",name,
+	      image ? "" : "*",
+#ifdef LCF_VIRTUAL
+	      descrs.d.descr[image].flags & FLAG_VMDEFAULT ? "@" :
+#endif
+	      ""
+	      );
 	    if (verbose >= 2) {
 	        if (descrs.d.descr[image].start.device & (LINEAR_FLAG|LBA32_FLAG)) {
 		   unsigned int sector;
@@ -652,6 +657,12 @@ static void show_images(char *map_file)
 	    printf("\n");
 	    if (verbose >= 1) {
 		flags = descrs.d.descr[image].flags;
+#ifdef LCF_VIRTUAL
+		if (flags & FLAG_VMDISABLE)
+		    printf("    Virtual Boot is disabled\n");
+		if (flags & FLAG_VMWARN)
+		    printf("    Warn on Virtual boot\n");
+#endif		
 		if ( !(flags & FLAG_PASSWORD) )
 		    printf("    No password\n");
 		else printf("    Password is required for %s\n",flags &
@@ -690,7 +701,7 @@ static void show_images(char *map_file)
 			  (unsigned long) descrs.d.descr[image].start_page*
 			  PAGE_SIZE);
 #else
-		    if (!(descrs.d.descr[image].flags & FLAG_LOADHI))
+		    if (!(flags & FLAG_LOADHI))
 			printf("    Kernel is loaded \"low\"\n");
 		    else printf("    Kernel is loaded \"high\"\n");
 #endif
@@ -1123,9 +1134,10 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
     }
 #endif
     if (linear && !nowarn) fprintf(errstd,
-    	"Warning: LINEAR is deprecated in favor of LBA32:  LINEAR specifies a 24-bit\n"
-	"  address; whereas LBA32 specifies a 32-bit address; use LINEAR only if you\n"
-	"  REALLY know what you are doing.\n");
+    	"Warning:  LINEAR is deprecated in favor of LBA32:  LINEAR specifies 24-bit\n"
+	"  disk addresses below the 1024 cylinder limit; LBA32 specifies 32-bit disk\n"
+	"  addresses not subject to cylinder limits on systems with EDD bios extensions;\n"
+	"  use LINEAR only if you are aware of its limitations.\n");
     if (verbose<0 && cfg_get_strg(cf_options,"verbose"))
 	verbose = to_number(cfg_get_strg(cf_options,"verbose"));
     if (verbose<0) verbose = 0;
