@@ -74,6 +74,7 @@ source directory.
 #define MAJOR_CISS	104 /* First CCISS Major 104-111 */
 #define MAJOR_IBM_iSER	112 /* IBM iSeries virtual disk */
 #define MAJOR_HPT370	114 /* HPT370 controller */
+#define MAJOR_EVMS	117 /* Enterprise Volume Management System */
 
 #define MAX_RAID	30	/* max number of RAID disks in a set */
 #define MAX_TOKEN	255 /* max device Token length */
@@ -197,7 +198,8 @@ source directory.
 #define EX_MAG_HL	0x4f4c494c  /* "LILO" */
 
 #define BIOS_MAX_DEVS	2	/* BIOS devices (2 fd, 2 hd) */
-#define BIOS_MAX_HEADS  255	/* 8 bits head number;  NOT 256 */
+#define BIOS_MAX_HEADS  256	/* 8 bits head number; really 255 */
+			/* but must account for oddball BIOS's that allow 256 */
 #define BIOS_MAX_CYLS   1024	/* 10 bits cylinder number */
 #define BIOS_MAX_SECS   63	/* 6 bits sector number (no, it's not 64) */
 
@@ -207,8 +209,8 @@ source directory.
 #define FLAG_RAID_DEFEAT  4	/* defeat finding this RAID boot record */
 #define FLAG_RAID_NOWRITE 8	/* defeat RAID writeback of command line */
 #define FLAG_NOBD	16	/* defeat BIOS data collection at boot time */
-#define FLAG_SAVE (FLAG_PROMPT|FLAG_NOBD)  /* All but raid flags */
 #define FLAG_LARGEMEM	32	/* BIOS has MoveExtMemBlk support for 386 */
+#define FLAG_SAVE (~(FLAG_RAID|FLAG_RAID_DEFEAT|FLAG_RAID_NOWRITE))  /* All but raid flags */
 
 /* these are the descriptor flags */
 #define FLAG_VGA	1	/* override VGA mode */
@@ -220,6 +222,11 @@ source directory.
 #define FLAG_FALLBACK	64	/* fallback command line exists */
 #define FLAG_PASSWORD	128	/* this image requires a password */
 #define FLAG_LOADHI	256	/* this kernel loads high (>=1Mb) */
+#ifdef LCF_VIRTUAL
+#define FLAG_VMDISABLE	512	/* unable to boot if virtual */
+#define FLAG_VMWARN	1024	/* warn on virtual boot */
+#define FLAG_VMDEFAULT	2048	/* this is the default vitual load */
+#endif
 
 #define VGA_NOCOVR	0x8000	/* VGA setting not overridden on command line */
 
