@@ -1,7 +1,7 @@
 /* partition.c  -  Partition table handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -17,6 +17,7 @@ source directory.
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <a.out.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <linux/unistd.h>
@@ -31,6 +32,7 @@ source directory.
 #include "geometry.h"
 #include "partition.h"
 #include "boot.h"
+#include "loader.h"
 
 
        
@@ -48,6 +50,69 @@ source directory.
        }
 
 
+
+
+
+/* identify partitions which would be destroyed if the boot block
+   is overwritten:
+   
+   known problems occur for:
+   	XFS
+   	NTFS
+   	DOS FAT (relocation will fix)
+
+*/
+int part_nowrite(char* device)
+{
+    int fd;
+    BOOT_SECTOR bs;
+    
+    int ret=0;	/* say ok, unless we recognize a problem partition */
+
+    if ((fd = open(device, O_RDONLY)) < 0) pdie("part_nowrite check:");
+    if (read(fd, bs.sector, sizeof(bs)) != SECTOR_SIZE) pdie("part_nowrite: read:");
+    
+/* check for XFS */
+    if (!strncmp("XFSB", bs.sector, 4)) ret=2;
+    
+/* check for NTFS */
+    else if (	!strncmp("NTFS", bs.par_d.system, 4)
+		|| strstr(bs.sector,"NTLDR")  ) ret=2;
+
+/* check for DOS FAT */
+    else if (
+	(bs.par_d.bpb.media_descriptor >= 0xF8 || bs.par_d.bpb.media_descriptor == 0xF0)
+	&& *(short*)bs.par_d.bpb.bytes_per_sector == SECTOR_SIZE
+	&& (bs.par_d.bpb.number_of_FATs==1 || bs.par_d.bpb.number_of_FATs==2)
+    /* sectors_per_cluster is a power of 2, meaning only 1 bit is on */
+	&& bs.par_d.bpb.sectors_per_cluster
+	&& (bs.par_d.bpb.sectors_per_cluster & (bs.par_d.bpb.sectors_per_cluster-1))==0
+				) ret=1;
+    
+/* check for SWAP -- last check, as 'bs' is overwritten */
+    else if (*(long*)bs.sector == 0xFFFFFFFEUL) {
+	if (lseek(fd, (PAGE_SIZE)-SECTOR_SIZE, SEEK_SET) != (PAGE_SIZE)-SECTOR_SIZE)
+	    pdie("part_nowrite lseek:");
+	if (SECTOR_SIZE != read(fd, bs.sector, sizeof(bs)) ) pdie("part_nowrite swap check:");
+	if (!strncmp(bs.sector+SECTOR_SIZE-10,"SWAPSPACE2",10)
+	    || !strncmp(bs.sector+SECTOR_SIZE-10,"SWAP-SPACE",10) ) ret=2;
+    }
+
+/* didn't recognize the superblock type, so assume it is okay */    
+    else ret=0;
+    
+    close(fd);
+
+/* check for override (-F) command line flag */
+    if (ret==2 && force_fs) {
+	fprintf(errstd, "WARNING: '-F' override used. Filesystem on  %s  may be destroyed.\n", device);
+	ret=0;
+    }
+    
+    return ret;
+}
+
+
 void part_verify(int dev_nr,int type)
 {
     GEOMETRY geo;
@@ -56,7 +121,7 @@ void part_verify(int dev_nr,int type)
     int fd, bck_file, part, size, lin_3d, cyl;
     unsigned long second, base;
     struct partition part_table[PART_MAX];
-    int mask, i, pe;
+    int mask, i, pe, Linux, dos;
     unsigned short boot_sig;
     
     if (!has_partitions(dev_nr) || !(mask = P_MASK(dev_nr)) || !(dev_nr & mask)
@@ -88,7 +153,7 @@ void part_verify(int dev_nr,int type)
     }
     i=5;
     while (i<=pe && base) {
-        if (llseek(fd, SECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
+        if (llseek(fd, LLSECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
             die("secondary llseek failed");
 	if (read(fd, part_table, sizeof(part_table)) != sizeof(part_table)) die("secondary read pt failed");
 	if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
@@ -99,13 +164,28 @@ void part_verify(int dev_nr,int type)
         part=0;
     }
 
-    if (type && part_table[part].sys_ind != PART_LINUX_MINIX &&
-	      part_table[part].sys_ind != PART_LINUX_NATIVE &&
-	      !is_extd_part(part_table[part].sys_ind) ) {
+    i = part_table[part].sys_ind;
+
+    Linux =   i == PART_LINUX_MINIX ||
+	      i == PART_LINUX_NATIVE ||
+	      is_extd_part(i);
+
+    i &= ~HIDDEN_OFF;
+    dos =     i == PART_DOS12 ||
+	      i == PART_DOS16_SMALL ||
+	      i == PART_DOS16_BIG ||
+	      i == PART_FAT32 ||
+	      i == PART_FAT32_LBA ||
+	      i == PART_FAT16_LBA ;
+
+    if (type && !Linux) {
 	fflush(stdout);
-	fprintf(errstd,"Device 0x%04X: Partition type 0x%02X does not seem "
-	    "suitable for a LILO boot sector\n",dev_nr,part_table[part].sys_ind);
-	if (!cfg_get_flag(cf_options,"ignore-table")) exit(1);
+	if (!dos || !nowarn)
+	    fprintf(errstd,"Device 0x%04X: Partition type 0x%02X does not seem "
+	      "suitable for a LILO boot sector\n",dev_nr,part_table[part].sys_ind);
+	if (dos && !nowarn)
+	    fprintf(errstd,"but DOS/Windows/OS2 FAT partitions are now usable.\n");
+	if (!dos && !cfg_get_flag(cf_options,"ignore-table")) exit(1);
     }
     cyl = part_table[part].cyl+((part_table[part].sector >> 6) << 8);
     lin_3d = (part_table[part].sector & 63)-1+(part_table[part].head+
@@ -115,20 +195,27 @@ void part_verify(int dev_nr,int type)
 	    part_table[part].start_sect && cyl != BIOS_MAX_CYLS-1)) &&
 	    !nowarn) {
 	fflush(stdout);
-	fprintf(errstd,"Device 0x%04X: Invalid partition table, %d%s entry\n",
+	fprintf(errstd,"Device 0x%04X: Inconsistent partition table, %d%s entry\n",
 	  dev_nr & ~mask,part+1,!part ? "st" : part == 1 ? "nd" : part ==
 	  2 ? "rd" : "th");
-	fprintf(errstd,"  3D address:     %d/%d/%d (%d)\n",part_table[part].
-	  sector & 63,part_table[part].head,cyl,lin_3d);
+	fprintf(errstd,"  CHS address in PT:  %d:%d:%d  -->  LBA (%d)\n",
+		cyl,
+		part_table[part].head,
+		part_table[part].sector & 63,
+		lin_3d);
 	cyl = part_table[part].start_sect/geo.sectors/geo.heads;
-	fprintf(errstd,"  Linear address: %d/%d/%d (%d)\n",part_table[part].
-	  sector = (part_table[part].start_sect % geo.sectors)+1,part_table
-	  [part].head = (part_table[part].start_sect/geo.sectors) %
-	  geo.heads,cyl,part_table[part].start_sect);
-	part_table[part].sector |= cyl >> 8;
+	fprintf(errstd,"  LBA address in PT:  %d  -->  CHS (%d:%d:%d)\n",
+		part_table[part].start_sect,
+		cyl,
+		part_table[part].head = (part_table[part].start_sect/geo.sectors) % geo.heads,
+		part_table[part].sector = (part_table[part].start_sect % geo.sectors)+1
+		);
+	if (cyl >= BIOS_MAX_CYLS) cyl = BIOS_MAX_CYLS-1;
+	part_table[part].sector |= (cyl >> 8)<<6;
 	part_table[part].cyl = cyl & 0xff;
 	if (!cfg_get_flag(cf_options,"fix-table") && !cfg_get_flag(cf_options,
-	  "ignore-table")) exit(1);
+	  "ignore-table")) die("Either FIX-TABLE or IGNORE-TABLE must be specified\n"
+			"If not sure, first try IGNORE-TABLE (-P ignore)");
 	if (test || cfg_get_flag(cf_options,"ignore-table"))
 	    fprintf(errstd,"The partition table is *NOT* being adjusted.\n");
 	else {
@@ -496,7 +583,10 @@ void do_activate(char *where, char *which)
 
 void do_install_mbr(char *where, char *what)
 {
-    int fd, nfd, i;
+    int fd, i;
+#ifndef LCF_BUILTIN
+    int nfd;
+#endif
     struct stat st;
     char buf[SECTOR_SIZE];
     
@@ -507,10 +597,12 @@ void do_install_mbr(char *where, char *what)
     if (st.st_rdev != (st.st_rdev & has_partitions(st.st_rdev)))
 	die("%s is not a master device with a primary parition table",where);
     if (read(fd,buf,SECTOR_SIZE) != SECTOR_SIZE) die("read %s: %s",where, strerror(errno));
-    
+#ifndef LCF_BUILTIN    
     if ((nfd=open(what,O_RDONLY)) < 0) die("Cannot open %s: %s",what,strerror(errno));
     if (read(nfd,buf,MAX_BOOT_SIZE) != MAX_BOOT_SIZE) die("read %s: %s",what,strerror(errno));
-    
+#else
+    memcpy(buf, Mbr.data, MAX_BOOT_SIZE);
+#endif    
     *(unsigned short*)&buf[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
     if (zflag) {
         char *p = buf+MAX_BOOT_SIZE;
@@ -530,7 +622,10 @@ void do_install_mbr(char *where, char *what)
 	if (write(fd,buf,SECTOR_SIZE) != SECTOR_SIZE)
 		die("write %s: %s",where,strerror(errno));
     }
-    close(fd); close(nfd);
+    close(fd);
+#ifndef LCF_BUILTIN
+    close(nfd);
+#endif
     printf("The Master Boot Record of  %s  has %sbeen updated.\n", where, test ? "*NOT* " : "");
     sync();
     exit(0);
