@@ -18,12 +18,22 @@ typedef struct _change_rule {
 
 #if 1
 
-#define SECTORSIZE ((long long)SECTOR_SIZE)
+#define LLSECTORSIZE ((long long)SECTOR_SIZE)
 
        loff_t llseek(unsigned int fd, loff_t offs, unsigned int whence);
 
 #endif
 
+int part_nowrite(char* device);
+/* identify partitions which would be destroyed if the boot block
+   is overwritten:
+   
+   known problems occur for:
+   	XFS
+   	NTFS
+   	DOS FAT (relocation will fix)
+*/
+
 #define is_extd_part(x) ((x)==PART_DOS_EXTD||(x)==PART_WIN_EXTD_LBA||(x)==PART_LINUX_EXTD)
 
 void part_verify(int dev_nr,int type);
