@@ -1,6 +1,6 @@
 /* probe.c -- BIOS probes */
 /*
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2002 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -217,6 +217,12 @@ static void do_help(void)
 #endif
 }
 
+/* diagnostic output */
+static void show_geom(char *who, int cyl, int head, int sect)
+{
+   fprintf(errstd, "    %s: %d cylinders, %d heads, %d sectors\n", who, cyl, head, sect);
+}
+
 
 /* get the old BIOS disk geometry */
 static int get_geom(unsigned int drive, struct disk_geom *geom)
@@ -355,12 +361,19 @@ static int get_geom(unsigned int drive, struct disk_geom *geom)
              geom->n_head != dp->heads) && !nowarn && !(warned[drive-0x80]&1) ) {
                 fprintf(errstd,"Warning: Int 0x13 function 8 and function 0x48 return different\n"
                                "head/sector geometries for BIOS drive 0x%02X\n", drive);
+                show_geom("fn 08", geom->n_cyl, geom->n_head, geom->n_sect);
+                show_geom("fn 48", dp->cylinders, dp->heads, dp->sectors);
 		warned[drive-0x80] |= 1;
 	    }
+
+/* prefer to return the fn 8 geometry */
+#if 0
          geom->n_cyl  = dp->cylinders;
          geom->n_head = dp->heads;
          geom->n_sect = dp->sectors;
-         geom->n_total_blocks = dp->total_sectors;
+#endif
+	 if (dp->total_sectors > geom->n_total_blocks)
+		geom->n_total_blocks = dp->total_sectors;
       }
 
    }
@@ -383,7 +396,7 @@ static int get_conv_mem(void)
 /* print the conventional memory size */
 static void do_ebda(void)
 {
-    int m, n;
+    int m, n, init;
     static char EBDA[]="Extended BIOS Data Area (EBDA)";
     
     m = get_conv_mem();
@@ -398,6 +411,8 @@ static void do_ebda(void)
     n = (select_loader()->size + SECTOR_SIZE - 1) / SECTOR_SIZE;
     n = m - (n+4+MAX_DESCR_SECS)*SECTOR_SIZE;
 #endif
+    init = (n - (MAX_SETUPSECS+1)*SECTOR_SIZE)>>4;
+    if (init > DEF_INITSEG) init = DEF_INITSEG;
 #if 0
     if (!(verbose>0)) return;
 #else
@@ -407,8 +422,8 @@ static void do_ebda(void)
     			FIRSTSEG<<4, FIRSTSEG<<4);
     printf("    The Second stage loader runs at:  0x%08X  (%04X:%04X)\n",
     			n, n>>4, n&15);
-    printf("    The kernel cmdline is passed at:  0x%08X  (%1X000:%04X)\n",
-    			m, m>>16, m&0xFFFF);
+    printf("    The kernel cmdline is passed at:  0x%08X  (%04X:%04X)\n",
+    			m, init, m-(init<<4));
 
 }
 
@@ -416,10 +431,13 @@ static void do_ebda(void)
 /* print the CHS geometry information for the specified disk */
 static void print_geom(int dr, struct disk_geom geom)
 {
-	 printf("    bios=0x%02x, cylinders=%d, heads=%d, sectors=%d\n", 
-		dr, geom.n_cyl, geom.n_head, geom.n_sect);
-	 if (geom.EDD_flags & EDD_PACKET)
-	    printf("\tEDD packet calls allowed\n");
+    char ch_ser[12] = { 0 };
+
+    if (geom.serial_no) sprintf(ch_ser, "%08X", geom.serial_no);
+    printf("    bios=0x%02x, cylinders=%d, heads=%d, sectors=%d\t%s\n", 
+	dr, geom.n_cyl, geom.n_head, geom.n_sect, ch_ser);
+    if (geom.EDD_flags & EDD_PACKET)
+    printf("\tEDD packet calls allowed\n");
 }
 
 
@@ -844,12 +862,22 @@ int bios_device(GEOMETRY *geo, int device)
     if (match != 1) bios1 = -1;
     else {	/* match == 1 */
 	get_geom(bios1, &bdata);
-	if (!nowarn &&
-	    (  (geo->sectors && geo->sectors!=bdata.n_sect)  ||
-	       (geo->heads && geo->heads!=bdata.n_head)  ) &&
-	    !(warned[bios1-0x80]&2) ) {
-	    fprintf(errstd,"Warning: Kernel & BIOS return differing CHS geometry for device 0x%02X\n", bios1);
-	    warned[bios1-0x80] |= 2;
+	if (  (geo->sectors && geo->sectors!=bdata.n_sect)  ||
+	       (geo->heads && geo->heads!=bdata.n_head)  )  {
+	    unsigned long nblocks = geo->cylinders * geo->heads * geo->sectors;
+
+	    if (!nowarn && !(warned[bios1-0x80]&2) ) {
+		fprintf(errstd,"Warning: Kernel & BIOS return differing head/sector geometries for device 0x%02X\n", bios1);
+		show_geom("Kernel", geo->cylinders, geo->heads, geo->sectors);
+		show_geom("  BIOS", bdata.n_cyl, bdata.n_head, bdata.n_sect);
+		warned[bios1-0x80] |= 2;
+	    }
+#if 1
+	    geo->sectors = bdata.n_sect;
+	    geo->heads = bdata.n_head;
+	    if (bdata.n_total_blocks > nblocks) nblocks = bdata.n_total_blocks;
+	    geo->cylinders = nblocks / (bdata.n_head*bdata.n_sect);
+#endif
 	}
     }
 
